use bevy::prelude::*;

mod map;
use crate::map::{Direction, example_map};

// use std::f32::consts::PI;

// Our camera
// also, the position of the party
#[derive(Component)]
struct BlobberCam;


const HEIGHT: f32 = 0.5;
const WALLOFFSET: f32 = 0.5;

fn grid2world(gridx: usize, gridy: usize) -> (f32, f32) {
    return (-(gridx as f32), gridy as f32);
}

fn wall(gridx: usize, gridy: usize,
        mesh: Handle<Mesh>,
        material: Handle<StandardMaterial>,
        side: Direction)
        -> PbrBundle {
    let (worldx, worldz) = grid2world(gridx, gridy);

    let (xoffset, zoffset, facing) = match side {
        Direction::North => (WALLOFFSET, 0., Vec3::NEG_X),
        Direction::South => (- WALLOFFSET, 0., Vec3::X),
        Direction::East  => (0., WALLOFFSET, Vec3::Z),
        Direction::West  => (0., - WALLOFFSET, Vec3::NEG_Z),
    };

    return PbrBundle {
        mesh: mesh,
        material: material,
        transform: Transform::from_xyz(worldx + xoffset,
                                       HEIGHT,
                                       worldz + zoffset)
            .looking_to(Vec3::Y, facing),
        ..default()
    };
}


fn main() {
    App::new()
        .insert_resource(Msaa::default())
        .add_plugins(DefaultPlugins)
        .add_systems(Startup, setup)
        .add_systems(Update, move_blobber)
        .add_systems(Update, bevy::window::close_on_esc)
        .run();
}

/// set up a simple 3D scene
fn setup(
    mut commands: Commands,
    asset_server: Res<AssetServer>,
    mut meshes: ResMut<Assets<Mesh>>,
    mut materials: ResMut<Assets<StandardMaterial>>,
) {
    let wall_texture_handle = asset_server.load("textures/this end up.png");

    let wall_material_handle = materials.add(StandardMaterial {
        base_color_texture: Some(wall_texture_handle.clone()),
        alpha_mode: AlphaMode::Blend,
        unlit: true,
        ..default()
    });

    let door_texture_handle = asset_server.load("textures/this end door.png");

    let door_material_handle = materials.add(StandardMaterial {
        base_color_texture: Some(door_texture_handle.clone()),
        alpha_mode: AlphaMode::Blend,
        unlit: true,
        ..default()
    });

    let wall_mesh = meshes.add(shape::Plane::from_size(1.0).into());

    // ground
    commands.spawn(PbrBundle {
        mesh: meshes.add(shape::Plane::from_size(45.0).into()),
        material: materials.add(Color::rgb(0.4, 0.4, 0.4).into()),
        ..default()
    });

    let gridmap = example_map();

    for gridx in 0..gridmap.len() {
        for gridy in 0..gridmap[gridx].len() {
            let gridcell = &gridmap[gridx][gridy];

            if gridcell.north_wall == "yes" {
                commands.spawn(wall(gridx, gridy,
                                    wall_mesh.clone(),
                                    wall_material_handle.clone(),
                                    Direction::North));
            } else if gridcell.north_wall == "door" {
                commands.spawn(wall(gridx, gridy,
                                    wall_mesh.clone(),
                                    door_material_handle.clone(),
                                    Direction::North));
            }

            if gridcell.south_wall == "yes" {
                commands.spawn(wall(gridx, gridy,
                                    wall_mesh.clone(),
                                    wall_material_handle.clone(),
                                    Direction::South));
            } else if gridcell.south_wall == "door" {
                commands.spawn(wall(gridx, gridy,
                                    wall_mesh.clone(),
                                    door_material_handle.clone(),
                                    Direction::South));
            }
            if gridcell.west_wall == "yes" {
                commands.spawn(wall(gridx, gridy,
                                    wall_mesh.clone(),
                                    wall_material_handle.clone(),
                                    Direction::West));
            } else if gridcell.west_wall == "door" {
                commands.spawn(wall(gridx, gridy,
                                    wall_mesh.clone(),
                                    door_material_handle.clone(),
                                    Direction::West));
            }
            if gridcell.east_wall == "yes" {
                commands.spawn(wall(gridx, gridy,
                                    wall_mesh.clone(),
                                    wall_material_handle.clone(),
                                    Direction::East));
            } else if gridcell.east_wall == "door" {
                commands.spawn(wall(gridx, gridy,
                                    wall_mesh.clone(),
                                    door_material_handle.clone(),
                                    Direction::East));
            }
        }
    }


    // light
    commands.spawn(PointLightBundle {
        point_light: PointLight {
            intensity: 1500.0,
            shadows_enabled: true,
            ..default()
        },
        transform: Transform::from_xyz(4.0, 8.0, 4.0),
        ..default()
    });


    commands
        .spawn((Camera3dBundle {
            transform: Transform::from_xyz(0., 0.5, 0.)
                .looking_to(Vec3::Z, Vec3::Y),
            ..default()
        },
                BlobberCam,
        ));
}


fn move_blobber(
    time: Res<Time>,
    keyboard: Res<Input<KeyCode>>,
    mut query: Query<(&BlobberCam, &mut Transform)>,
) {
    for (_camera, mut transform) in query.iter_mut() {
        let forward = transform.forward();
        if keyboard.pressed(KeyCode::Up) {
            // move forward
            transform.translation += forward * time.delta_seconds();
        } else if keyboard.pressed(KeyCode::Left) {
            // turn left
            transform.rotate_y(time.delta_seconds());
        } else if keyboard.pressed(KeyCode::Right) {
            // turn right
            transform.rotate_y(- time.delta_seconds());
        }
    }
}
